package xyz.zhouxy.plusone.domain;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * 实体
 *
 * <p>
 * DDD 中的实体，带有 ID。
 * </p>
 *
 * <p>
 * 维护一个 {@link DomainEvent} 的列表，持久化时可将其中的领域事件进行发布。
 * </p>
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 * @see IValueObject
 * @see AggregateRoot
 */
public abstract class Entity<ID extends Serializable> {

    private final List<DomainEvent> domainEvents = new ArrayList<>();

    public abstract Optional<ID> getId();

    protected void addDomainEvent(DomainEvent domainEvent) {
        domainEvents.add(domainEvent);
    }
}
