package xyz.zhouxy.plusone.jdbc;

import java.util.List;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.jdbc.core.convert.JdbcCustomConversions;
import org.springframework.data.jdbc.repository.QueryMappingConfiguration;
import org.springframework.data.jdbc.repository.config.AbstractJdbcConfiguration;
import org.springframework.data.jdbc.repository.config.DefaultQueryMappingConfiguration;

import xyz.zhouxy.plusone.jdbc.converter.EnumToOrdinalConverter;

/**
 * JDBC 配置
 *
 * <p>
 * 配置了类型转换器
 * </p>
 *
 * <p>
 * 这里声明了 {@link DefaultQueryMappingConfiguration} 的实例为 Spring bean。
 * 在其它配置类中注入该 bean，就可以向其中添加
 * {@link org.springframework.jdbc.core.RowMapper}
 * 供 Spring Data JDBC 查询时使用。如下所示：
 *
 * <pre>
 * {@code @Configuration}
 * class CustomConfig {
 *     public CustomConfig(DefaultQueryMappingConfiguration rowMappers) {
 *         rowMappers.registerRowMapper(Person.class, new PersonRowMapper());
 *     }
 * }
 * </pre>
 * </p>
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Configuration
public class JdbcConfig extends AbstractJdbcConfiguration {

    @Override
    public JdbcCustomConversions jdbcCustomConversions() {
        return new JdbcCustomConversions(List.of(EnumToOrdinalConverter.INSTANCE));
    }

    @Bean
    QueryMappingConfiguration rowMappers() {
        return new DefaultQueryMappingConfiguration();
    }
}
