package xyz.zhouxy.plusone.jdbc;

import java.io.Serializable;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.stream.Stream;

import javax.annotation.Nonnull;

import org.springframework.jdbc.core.ResultSetExtractor;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;

import xyz.zhouxy.plusone.domain.Entity;

public abstract class JdbcEntityDaoSupport<T extends Entity<ID>, ID extends Serializable> {
    protected final NamedParameterJdbcTemplate jdbc;

    protected RowMapper<T> rowMapper;
    protected ResultSetExtractor<T> resultSetExtractor;

    protected JdbcEntityDaoSupport(@Nonnull NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        this.jdbc = namedParameterJdbcTemplate;
        this.rowMapper = (ResultSet rs, int rowNum) -> mapRow(rs);
        this.resultSetExtractor = (ResultSet rs) -> rs.next() ? mapRow(rs) : null;
    }

    protected final T queryForObject(String sql) {
        return this.jdbc.query(sql, this.resultSetExtractor);
    }

    protected final T queryForObject(String sql, SqlParameterSource paramSource) {
        return this.jdbc.query(sql, paramSource, this.resultSetExtractor);
    }

    protected final List<T> queryForList(String sql) {
        return this.jdbc.query(sql, this.rowMapper);
    }

    protected final List<T> queryForList(String sql, SqlParameterSource parameterSource) {
        return this.jdbc.query(sql, parameterSource, this.rowMapper);
    }

    protected final Stream<T> queryForStream(String sql, SqlParameterSource parameterSource) {
        return this.jdbc.queryForStream(sql, parameterSource, this.rowMapper);
    }

    protected final <E> Stream<E> queryForStream(String sql, SqlParameterSource parameterSource, Class<E> elementType) {
        return this.jdbc.queryForList(sql, parameterSource, elementType).stream();
    }

    protected final boolean queryExists(String sql, SqlParameterSource parameterSource) {
        Boolean isExists = this.jdbc.query(sql, parameterSource, ResultSet::next);
        return Boolean.TRUE.equals(isExists);
    }

    protected final int update(String sql, SqlParameterSource parameterSource) {
        return this.jdbc.update(sql, parameterSource);
    }

    protected abstract T mapRow(ResultSet rs) throws SQLException;

    protected void setRowMapper(@Nonnull RowMapper<T> rowMapper) {
        this.rowMapper = rowMapper;
    }

    protected void setResultSetExtractor(@Nonnull ResultSetExtractor<T> resultSetExtractor) {
        this.resultSetExtractor = resultSetExtractor;
    }
}
