package xyz.zhouxy.plusone.jdbc;

import java.io.Serializable;

import javax.annotation.Nonnull;

import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;

import xyz.zhouxy.plusone.domain.AggregateRoot;
import xyz.zhouxy.plusone.domain.IRepository;

public abstract class JdbcRepositorySupport<T extends AggregateRoot<ID>, ID extends Serializable>
        extends JdbcEntityDaoSupport<T, ID>
        implements IRepository<T, ID> {

    protected JdbcRepositorySupport(@Nonnull NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        super(namedParameterJdbcTemplate);
    }

    protected abstract void doDelete(@Nonnull T entity);

    protected abstract T doFindById(@Nonnull ID id);

    protected abstract T doInsert(@Nonnull T entity);

    protected abstract T doUpdate(@Nonnull T entity);

    @Override
    public final void delete(T entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Cannot delete null.");
        }
        doDelete(entity);
    }

    @Override
    public final T find(ID id) {
        if (id == null) {
            throw new IllegalArgumentException("Id cannot be null.");
        }
        return doFindById(id);
    }

    @Override
    public final T save(T entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Cannot save null.");
        }
        return entity.getId().isPresent() ? doUpdate(entity) : doInsert(entity);
    }

    protected abstract SqlParameterSource generateParamSource(ID id, @Nonnull T entity);

    protected final SqlParameterSource generateParamSource(@Nonnull T entity) {
        return generateParamSource(entity.getId().orElseThrow(), entity);
    }
}
