package xyz.zhouxy.plusone.jdbc.common;

import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * 查询结果处理
 *
 * <p>
 * 通过在 {@link #map(ResultSet)} 中配置 {@link ResultSet} 到对象的映射，
 * 可将 {@link #rowMapper(ResultSet, int)} 的方法应用，
 * 直接当成 {@link org.springframework.jdbc.core.RowMapper} 对象传给
 * {@link org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate}
 * 的查询方法，
 * 或者在 Spring Data JDBC 的配置中使用。
 * </p>
 *
 * <p>
 * 在
 * {@link org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate}
 * 的 query(String, java.util.Map, ResultSetExtractor)
 * 和 query(String, SqlParameterSource, ResultSetExtractor)
 * 两个方法执行时，ResultSetExtractor 中需要执行一次 {@link ResultSet#next()} 判断是否查询到数据，
 * 如果查询到数据再执行查询结果的实例化。
 * {@link #resultSetExtractor(ResultSet)} 封装了这个过程，
 * 和 {@link #rowMapper(ResultSet, int)}一样，
 * 直接把方法引用当成 {@link org.springframework.jdbc.core.ResultSetExtractor} 的对象即可。
 * </p>
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 * @see org.springframework.data.jdbc.repository.config.DefaultQueryMappingConfiguration
 */
@FunctionalInterface
public interface ResultMapper<T> {

    T map(ResultSet resultSet) throws SQLException;

    default T rowMapper(ResultSet resultSet, int rowNum) throws SQLException {
        return map(resultSet);
    }

    default T resultSetExtractor(ResultSet resultSet) throws SQLException {
        return resultSet.next() ? map(resultSet) : null;
    }
}
