package xyz.zhouxy.plusone.jdbc.converter;

import org.springframework.core.convert.converter.Converter;
import org.springframework.data.convert.ReadingConverter;

/**
 * 序号枚举转换器
 * <p>
 * 向 {@link org.springframework.data.jdbc.core.convert.JdbcCustomConversions}
 * 中添加类型转换器，
 * 如：
 *
 * <pre>
 * return new JdbcCustomConversions(List.of(new OrdinalToEnumConverter<SystemStatus>()));
 * </pre>
 * </p>
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 *
 * @see Converter
 * @see org.springframework.data.jdbc.core.convert.JdbcCustomConversions
 */
@ReadingConverter
public class OrdinalToEnumConverter<E extends Enum<E>> implements Converter<Integer, Enum<E>> {

    private final Class<E> type;
    private final E[] constants;

    public OrdinalToEnumConverter(Class<E> type) {
        this.type = type;
        this.constants = type.getEnumConstants();
    }

    @Override
    public Enum<E> convert(Integer ordinal) {
        try {
            return constants[ordinal];
        } catch (ArrayIndexOutOfBoundsException exception) {
            throw new IllegalArgumentException(
                    String.format("Cannot convert %d to %s by ordinal value.", ordinal, type.getSimpleName()),
                    exception);
        }
    }
}
