package xyz.zhouxy.plusone.redis;

import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Repository;

import java.util.concurrent.TimeUnit;

/**
 * 使用 Redis 存放临时字符串
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Repository
public class RedisStrCacheDAO implements StrCacheDAO {

    private final StringRedisTemplate template;

    public RedisStrCacheDAO(StringRedisTemplate template) {
        this.template = template;
    }

    @Override
    public void setValue(String key, String value, long timeout, TimeUnit unit) {
        ValueOperations<String, String> ops = template.opsForValue();
        ops.set(key, value, timeout, unit);
    }

    @Override
    public String getValue(String key) {
        ValueOperations<String, String> ops = this.template.opsForValue();
        return ops.get(key);
    }

    @Override
    public String getValueAndDelete(String key) {
        ValueOperations<String, String> ops = this.template.opsForValue();
        String value = ops.get(key);
        template.delete(key);
        return value;
    }
}
