package xyz.zhouxy.plusone.web.config;

import org.springframework.context.annotation.Configuration;
import org.springframework.core.convert.converter.Converter;
import org.springframework.core.convert.converter.ConverterFactory;
import org.springframework.format.FormatterRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

/**
 * Spring MVC 配置。
 *
 * <p>
 * 将与前后端交互的数据中的枚举转换为其整数值
 * </p>
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Configuration
public class WebEnumMapConfig implements WebMvcConfigurer {
    @Override
    public void addFormatters(FormatterRegistry registry) {
        registry.addConverterFactory(new IntToEnumConverterFactory());
        registry.addConverterFactory(new StringToEnumConverterFactory());
    }
}


class IntToEnumConverterFactory implements ConverterFactory<Integer, Enum<?>> {
    @Override
    public <T extends Enum<?>> Converter<Integer, T> getConverter(Class<T> targetType) {
        return (Integer source) -> {
            try {
                T[] values = targetType.getEnumConstants();
                return values[source];
            } catch (IndexOutOfBoundsException e) {
                throw new EnumConstantNotPresentException(targetType, Integer.toString(source));
            }
        };
    }
}

class StringToEnumConverterFactory implements ConverterFactory<String, Enum<?>> {
    @Override
    public <T extends Enum<?>> Converter<String, T> getConverter(Class<T> targetType) {
        return (String source) -> {
            int index = Integer.parseInt(source);
            try {
                T[] values = targetType.getEnumConstants();
                return values[index];
            } catch (IndexOutOfBoundsException e) {
                throw new EnumConstantNotPresentException(targetType, Integer.toString(index));
            }
        };
    }
}
