package xyz.zhouxy.plusone.system.application.common.util;

import javax.annotation.Nullable;

import xyz.zhouxy.plusone.exception.InvalidInputException;
import xyz.zhouxy.plusone.system.domain.model.account.Email;
import xyz.zhouxy.plusone.system.domain.model.account.MobilePhone;
import xyz.zhouxy.plusone.system.domain.model.account.Principal;
import xyz.zhouxy.plusone.system.domain.model.account.Username;

/**
 * 根据字面值，判断并生成 {@link Principal} 值对象。
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 * @see Principal
 * @see Username
 * @see Email
 * @see MobilePhone
 * @see InvalidInputException
 */
public class PrincipalUtil {

    private PrincipalUtil() {
        throw new IllegalStateException("Utility class");
    }

    public static PrincipalType getPrincipalType(@Nullable String principal) {
        if (principal == null) {
            throw new IllegalArgumentException("principal 不能为空");
        }
        PrincipalType[] principalTypes = PrincipalType.values();
        for (var principalType : principalTypes) {
            if (principal.matches(principalType.getRegex())) {
                return principalType;
            }
        }
        throw InvalidInputException.unsupportedPrincipalTypeException();
    }

    public static Principal getPrincipal(@Nullable String principal) {
        PrincipalType principalType = getPrincipalType(principal);
        if (principalType == PrincipalType.EMAIL) {
            return Email.of(principal);
        }
        if (principalType == PrincipalType.MOBILE_PHONE) {
            return MobilePhone.of(principal);
        }
        if (principalType == PrincipalType.USERNAME) {
            return Username.of(principal);
        }
        throw InvalidInputException.unsupportedPrincipalTypeException();
    }

    public static Principal getEmailOrMobilePhone(@Nullable String principal) {
        PrincipalType principalType = getPrincipalType(principal);
        if (principalType == PrincipalType.EMAIL) {
            return Email.of(principal);
        }
        if (principalType == PrincipalType.MOBILE_PHONE) {
            return MobilePhone.of(principal);
        }
        throw InvalidInputException.unsupportedPrincipalTypeException("输入邮箱地址或手机号");
    }
}
