package xyz.zhouxy.plusone.system.application.controller;

import static xyz.zhouxy.plusone.system.constant.AuthLogic.adminAuthLogic;
import static xyz.zhouxy.plusone.util.RestfulResult.success;

import java.util.List;

import javax.validation.Valid;

import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import xyz.zhouxy.plusone.system.application.query.params.AccountQueryParams;
import xyz.zhouxy.plusone.system.application.service.AccountManagementService;
import xyz.zhouxy.plusone.system.application.service.command.CreateAccountCommand;
import xyz.zhouxy.plusone.system.application.service.command.UpdateAccountCommand;
import xyz.zhouxy.plusone.util.AssertResult;
import xyz.zhouxy.plusone.util.RestfulResult;

/**
 * 账号管理
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@RestController
@RequestMapping("sys/account")
public class AccountManagementController {

    private final AccountManagementService service;

    public AccountManagementController(AccountManagementService service) {
        this.service = service;
    }

    @PostMapping
    public RestfulResult createAccount(@RequestBody @Valid CreateAccountCommand command) {
        adminAuthLogic.checkLogin();
        adminAuthLogic.checkPermission("sys-account-create");
        service.createAccount(command);
        return success();
    }

    @DeleteMapping
    public RestfulResult deleteAccounts(@RequestBody List<Long> ids) {
        adminAuthLogic.checkLogin();
        adminAuthLogic.checkPermission("sys-account-delete");
        service.deleteAccounts(ids);
        return success();
    }

    @PatchMapping("{id}")
    public RestfulResult updateAccountInfo(
            @PathVariable("id") Long id,
            @RequestBody @Valid UpdateAccountCommand command) {
        adminAuthLogic.checkLogin();
        adminAuthLogic.checkPermission("sys-account-update");
        service.updateAccountInfo(id, command);
        return success();
    }

    @GetMapping("query")
    public RestfulResult queryAccountOverviewList(AccountQueryParams queryParams) {
        adminAuthLogic.checkLogin();
        adminAuthLogic.checkPermission("sys-account-list");
        var accountOverviewList = service.queryAccountOverviewList(queryParams);
        return success("查询成功", accountOverviewList);
    }

    @GetMapping("{accountId}")
    public RestfulResult queryAccountDetails(@PathVariable("accountId") Long accountId) {
        adminAuthLogic.checkLogin();
        adminAuthLogic.checkPermission("sys-account-details");
        var accountDetails = service.queryAccountDetails(accountId);
        AssertResult.nonNull(accountDetails);
        return success("查询成功", accountDetails);
    }
}
