package xyz.zhouxy.plusone.system.application.controller;

import static xyz.zhouxy.plusone.system.constant.AuthLogic.adminAuthLogic;
import static xyz.zhouxy.plusone.util.RestfulResult.success;

import java.util.List;

import javax.validation.Valid;

import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import xyz.zhouxy.plusone.system.application.query.params.DictQueryParams;
import xyz.zhouxy.plusone.system.application.service.DictManagementService;
import xyz.zhouxy.plusone.system.application.service.command.CreateDictCommand;
import xyz.zhouxy.plusone.system.application.service.command.UpdateDictCommand;
import xyz.zhouxy.plusone.util.RestfulResult;

/**
 * 数据字典管理
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@RestController
@RequestMapping("sys/dict")
public class DictManagementController {

    private final DictManagementService service;

    public DictManagementController(DictManagementService service) {
        this.service = service;
    }

    @PostMapping
    public RestfulResult createDict(@RequestBody @Valid CreateDictCommand command) {
        adminAuthLogic.checkPermission("sys-dict-create");
        service.createDict(command);
        return success();
    }

    @DeleteMapping
    public RestfulResult deleteDicts(@RequestBody List<Long> ids) {
        adminAuthLogic.checkPermission("sys-dict-delete");
        service.deleteDicts(ids);
        return success();
    }

    @PatchMapping("{id}")
    public RestfulResult updateDict(
            @PathVariable("id") Long id,
            @RequestBody @Valid UpdateDictCommand command) {
        adminAuthLogic.checkPermission("sys-dict-update");
        service.updateDict(id, command);
        return success();
    }

    @GetMapping("{dictId}")
    public RestfulResult findDictDetails(@PathVariable("dictId") Long dictId) {
        adminAuthLogic.checkPermission("sys-dict-details");
        var dictDetails = service.findDictDetails(dictId);
        return success("查询成功", dictDetails);
    }

    @GetMapping("all")
    public RestfulResult loadAllDicts() {
        adminAuthLogic.checkPermissionAnd("sys-dict-list", "sys-dict-details");
        var dicts = service.loadAllDicts();
        return success("查询成功", dicts);
    }

    @GetMapping("query")
    public RestfulResult queryDictOverviewList(@Valid DictQueryParams queryParams) {
        adminAuthLogic.checkPermission("sys-dict-list");
        var dicts = service.queryDictOverviewList(queryParams);
        return success("查询成功", dicts);
    }
}
