package xyz.zhouxy.plusone.system.application.controller;

import static xyz.zhouxy.plusone.system.constant.AuthLogic.adminAuthLogic;
import static xyz.zhouxy.plusone.util.RestfulResult.success;

import javax.validation.Valid;

import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import xyz.zhouxy.plusone.system.application.service.MenuManagementService;
import xyz.zhouxy.plusone.system.application.service.command.CreateMenuCommand;
import xyz.zhouxy.plusone.system.application.service.command.UpdateMenuCommand;
import xyz.zhouxy.plusone.util.RestfulResult;

/**
 * 菜单管理
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@RestController
@RequestMapping("sys/menu")
public class MenuManagementController {

    private final MenuManagementService service;

    public MenuManagementController(MenuManagementService service) {
        this.service = service;
    }

    // ==================== create ====================
    @PostMapping
    public RestfulResult createMenu(@RequestBody @Valid CreateMenuCommand command) {
        adminAuthLogic.checkPermission("sys-menu-create");
        service.createMenu(command);
        return success();
    }

    // ==================== delete ====================
    @DeleteMapping("{id}")
    public RestfulResult deleteMenu(@PathVariable("id") Long id) {
        adminAuthLogic.checkPermission("sys-menu-delete");
        service.deleteMenu(id);
        return success();
    }

    // ==================== update ====================
    @PatchMapping("{id}")
    public RestfulResult updateMenu(
            @PathVariable("id") Long id,
            @RequestBody @Valid UpdateMenuCommand command) {
        adminAuthLogic.checkPermission("sys-menu-update");
        service.updateMenu(id, command);
        return success();
    }

    // ==================== query ====================
    @GetMapping("{id}")
    public RestfulResult findById(@PathVariable("id") Long id) {
        adminAuthLogic.checkPermission("sys-menu-details");
        var result = service.findById(id);
        return RestfulResult.success("查询成功", result);
    }

    @GetMapping("queryByAccountId")
    public RestfulResult queryByAccountId(@RequestParam Long accountId) {
        adminAuthLogic.checkPermission("sys-menu-details");
        var result = service.queryByAccountId(accountId);
        return success("查询成功", result);
    }

    @GetMapping("queryByRoleId")
    public RestfulResult queryByRoleId(@RequestParam Long roleId) {
        adminAuthLogic.checkPermission("sys-menu-details");
        var result = service.queryByRoleId(roleId);
        return success("查询成功", result);
    }
}
