package xyz.zhouxy.plusone.system.application.controller;

import javax.validation.Valid;
import static xyz.zhouxy.plusone.system.constant.AuthLogic.adminAuthLogic;

import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import xyz.zhouxy.plusone.system.application.query.params.RoleQueryParams;
import xyz.zhouxy.plusone.system.application.service.RoleManagementService;
import xyz.zhouxy.plusone.system.application.service.command.CreateRoleCommand;
import xyz.zhouxy.plusone.system.application.service.command.UpdateRoleCommand;
import xyz.zhouxy.plusone.util.RestfulResult;

/**
 * 角色管理服务
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@RestController
@RequestMapping("sys/role")
public class RoleManagementController {

    private final RoleManagementService service;

    public RoleManagementController(RoleManagementService service) {
        this.service = service;
    }

    @PostMapping
    public RestfulResult createRole(@RequestBody @Valid CreateRoleCommand command) {
        adminAuthLogic.checkPermission("sys-role-create");
        service.createRole(command);
        return RestfulResult.success();
    }

    @PatchMapping
    public RestfulResult updateRole(@RequestBody @Valid UpdateRoleCommand command) {
        adminAuthLogic.checkPermission("sys-role-update");
        service.updateRole(command);
        return RestfulResult.success();
    }

    @DeleteMapping("{id}")
    public RestfulResult delete(@PathVariable("id") Long id) {
        adminAuthLogic.checkPermission("sys-role-delete");
        service.delete(id);
        return RestfulResult.success();
    }

    @GetMapping("exists")
    public RestfulResult exists(@RequestParam("id") Long id) {
        adminAuthLogic.checkPermissionOr("sys-role-list", "sys-role-details");
        var isExists = service.exists(id);
        return RestfulResult.success(isExists ? "存在" : "不存在", isExists);
    }

    @GetMapping("{id}")
    public RestfulResult findById(@PathVariable("id") Long id) {
        adminAuthLogic.checkPermission("sys-role-details");
        var result = service.findById(id);
        return RestfulResult.success("查询成功", result);
    }

    @GetMapping("query")
    public RestfulResult query(RoleQueryParams params) {
        adminAuthLogic.checkPermission("sys-role-list");
        var result = service.query(params);
        return RestfulResult.success("查询成功", result);
    }
}
