package xyz.zhouxy.plusone.system.application.exception.handler;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import cn.dev33.satoken.exception.DisableServiceException;
import cn.dev33.satoken.exception.SameTokenInvalidException;
import cn.dev33.satoken.exception.NotBasicAuthException;
import cn.dev33.satoken.exception.NotLoginException;
import cn.dev33.satoken.exception.NotPermissionException;
import cn.dev33.satoken.exception.NotRoleException;
import cn.dev33.satoken.exception.NotSafeException;
import cn.dev33.satoken.exception.SaTokenException;
import lombok.extern.slf4j.Slf4j;
import xyz.zhouxy.plusone.exception.handler.BaseExceptionHandler;
import xyz.zhouxy.plusone.util.RestfulResult;

/**
 * Sa-Token 异常处理器
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@RestControllerAdvice
@Slf4j
public class SaTokenExceptionHandler extends BaseExceptionHandler {

    public SaTokenExceptionHandler(ExceptionInfoHolder exceptionInfoHolder) {
        super(exceptionInfoHolder);
        set(NotPermissionException.class, 4030103, "会话未能通过权限认证", HttpStatus.FORBIDDEN);
        set(NotRoleException.class, 4030103, "会话未能通过角色认证", HttpStatus.FORBIDDEN);
        set(DisableServiceException.class, 4030202, "账号指定服务已被封禁", HttpStatus.FORBIDDEN);
        set(SameTokenInvalidException.class, 4030400, "提供的 Same-Token 无效", HttpStatus.UNAUTHORIZED);
        set(NotBasicAuthException.class, 4030000, "会话未能通过 Http Basic 认证", HttpStatus.UNAUTHORIZED);
        set(NotSafeException.class, 4020300, "会话未能通过二级认证", HttpStatus.UNAUTHORIZED);
        set(NotLoginException.class,
                4020400,
                e -> switch (((NotLoginException) e).getType()) {
                    case NotLoginException.NOT_TOKEN -> "未提供 Token";
                    case NotLoginException.INVALID_TOKEN -> "Token 无效";
                    case NotLoginException.TOKEN_TIMEOUT -> "Token 已过期";
                    case NotLoginException.BE_REPLACED -> "Token 已被顶下线";
                    case NotLoginException.KICK_OUT -> "Token 已被踢下线";
                    default -> "当前会话未登录";
                },
                HttpStatus.UNAUTHORIZED);
        set(SaTokenException.class, 4020300, "未通过身份认证或权限认证", HttpStatus.FORBIDDEN);
    }

    @ExceptionHandler(SaTokenException.class)
    public ResponseEntity<RestfulResult> handleSaTokenException(SaTokenException e) {
        log.error(e.getMessage(), e);
        return buildExceptionResponse(e);
    }
}
