package xyz.zhouxy.plusone.system.application.query.result;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;

import com.fasterxml.jackson.annotation.JsonInclude;

import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import xyz.zhouxy.plusone.domain.IWithOrderNumber;
import xyz.zhouxy.plusone.system.domain.model.menu.Action;
import xyz.zhouxy.plusone.system.domain.model.menu.Menu;
import xyz.zhouxy.plusone.system.domain.model.menu.Menu.MenuType;

/**
 * 菜单信息
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@NoArgsConstructor
@JsonInclude(JsonInclude.Include.NON_NULL)
public class MenuViewObject implements IWithOrderNumber {

    @Getter
    @Setter
    Integer type;

    @Getter
    @Setter
    String typeName;

    @Getter
    @Setter
    Long id;
    @Getter
    @Setter
    Long parentId;

    @Getter
    @Setter
    String name;
    // 若 type 为 MENU_ITEM 且 path 以 http:// 或 https:// 开头则被识别为外链
    @Getter
    @Setter
    String path;
    @Getter
    @Setter
    String title;
    @Getter
    @Setter
    String icon;
    @Getter
    @Setter
    boolean hidden;
    @Getter
    @Setter
    int orderNumber;
    @Getter
    @Setter
    Integer status;
    @Getter
    @Setter
    String remarks;

    // MENU_ITEM
    @Getter
    @Setter
    String component;
    @Getter
    @Setter
    Boolean cache;
    @Getter
    @Setter
    String resource;
    @Getter
    @Setter
    List<Action> actions;

    // MENU_LIST
    List<MenuViewObject> children;

    public void addChild(MenuViewObject child) {
        if (this.children == null) {
            this.children = new ArrayList<>();
        }
        this.children.add(child);
    }

    public void addChildren(Collection<MenuViewObject> children) {
        if (this.children == null) {
            this.children = new ArrayList<>();
        }
        this.children.addAll(children);
    }

    public static MenuViewObject of(Menu menu) {
        var viewObject = new MenuViewObject();
        viewObject.type = menu.getType().ordinal();
        viewObject.typeName = menu.getType().name();
        viewObject.id = menu.getId().orElseThrow();
        viewObject.parentId = menu.getParentId();
        viewObject.name = menu.getName();
        viewObject.path = menu.getPath();
        viewObject.title = menu.getTitle();
        viewObject.icon = menu.getIcon();
        viewObject.hidden = menu.isHidden();
        viewObject.orderNumber = menu.getOrderNumber();
        viewObject.status = menu.getStatus().getValue();
        viewObject.remarks = menu.getRemarks();
        if (viewObject.type == MenuType.MENU_ITEM.ordinal()) {
            viewObject.component = menu.getComponent();
            viewObject.cache = menu.getCache();
            viewObject.resource = menu.getResource();
            viewObject.actions = menu.getActions();
        }
        return viewObject;
    }

    public List<MenuViewObject> getChildren() {
        return Objects.nonNull(this.children)
                ? this.children
                        .stream()
                        .sorted(Comparator.comparing(IWithOrderNumber::getOrderNumber))
                        .toList()
                : null;
    }
}
