package xyz.zhouxy.plusone.system.application.service;

import static xyz.zhouxy.plusone.system.constant.AuthLogic.*;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.springframework.stereotype.Service;

import cn.dev33.satoken.session.SaSession;
import cn.dev33.satoken.stp.StpInterface;
import cn.dev33.satoken.stp.StpUtil;
import xyz.zhouxy.plusone.system.domain.model.menu.Action;
import xyz.zhouxy.plusone.system.domain.model.menu.MenuRepository;
import xyz.zhouxy.plusone.system.domain.model.role.ActionRef;
import xyz.zhouxy.plusone.system.domain.model.role.Role;
import xyz.zhouxy.plusone.system.domain.model.role.RoleRepository;

@Service
public class AuthService implements StpInterface {

    private RoleRepository roleRepository;
    private MenuRepository menuRepository;

    public AuthService(RoleRepository roleRepository, MenuRepository menuRepository) {
        this.roleRepository = roleRepository;
        this.menuRepository = menuRepository;
    }

    @Override
    public List<String> getPermissionList(Object loginId, String loginType) {
        Collection<Role> roles = getRoleList(loginId);
        Set<Long> permissionIds = new HashSet<>();
        roles.forEach(role -> permissionIds.addAll(
                role.getPermissions()
                        .stream()
                        .map(ActionRef::actionId)
                        .toList()));
        List<String> permValList = menuRepository.findPermissionsByIdIn(permissionIds)
                .stream().map(Action::value).toList();
        return permValList;
    }

    @Override
    public List<String> getRoleList(Object loginId, String loginType) {
        SaSession session = switch (loginType) {
            case ADMIN_LOGIN_TYPE -> adminAuthLogic.getSessionByLoginId(loginId);
            case USER_LOGIN_TYPE -> userAuthLogic.getSessionByLoginId(loginId);
            default -> StpUtil.getSessionByLoginId(loginId);
        };
        return session.get("RoleList",
                () -> getRoleList(loginId).stream().map(Role::getIdentifier).toList());
    }

    private Collection<Role> getRoleList(Object loginId) {
        return roleRepository.findByAccountId(Long.valueOf((String) loginId));
    }
}
