package xyz.zhouxy.plusone.system.application.service;

import java.util.List;
import java.util.Objects;

import javax.validation.Valid;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import xyz.zhouxy.plusone.system.application.query.DictQueries;
import xyz.zhouxy.plusone.system.application.query.params.DictQueryParams;
import xyz.zhouxy.plusone.system.application.query.result.DictOverview;
import xyz.zhouxy.plusone.system.application.service.command.CreateDictCommand;
import xyz.zhouxy.plusone.system.application.service.command.UpdateDictCommand;
import xyz.zhouxy.plusone.system.domain.model.dict.Dict;
import xyz.zhouxy.plusone.system.domain.model.dict.DictRepository;

/**
 * 数据字典管理
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Service
@Transactional
public class DictManagementService {

    private final DictRepository dictRepository;
    private final DictQueries dictQueries;

    public DictManagementService(DictRepository dictRepository, DictQueries dictQueries) {
        this.dictRepository = dictRepository;
        this.dictQueries = dictQueries;
    }

    public void createDict(@Valid CreateDictCommand command) {
        var dictToSave = Dict.newInstance(command.getDictType(),
                command.getDictLabel(),
                command.getKeyLabelMap());
        dictRepository.save(dictToSave);
    }

    public void deleteDicts(List<Long> ids) {
        Dict dictToDelete;
        for (Long id : ids) {
            dictToDelete = dictRepository.find(id);
            dictRepository.delete(dictToDelete);
        }
    }

    public void updateDict(Long id, @Valid UpdateDictCommand command) {
        Assert.isTrue(Objects.equals(id, command.getId()), "id 不匹配");
        Dict dictToUpdate = dictRepository.find(command.getId());
        dictToUpdate.updateDict(command.getDictType(), command.getDictLabel(), command.getKeyLabelMap());
        dictRepository.save(dictToUpdate);
    }

    @Transactional(propagation = Propagation.SUPPORTS)
    public Dict findDictDetails(Long dictId) {
        return dictRepository.find(dictId);
    }

    @Transactional(propagation = Propagation.SUPPORTS)
    public List<Dict> loadAllDicts() {
        return dictRepository.findAll();
    }

    @Transactional(propagation = Propagation.SUPPORTS)
    public List<DictOverview> queryDictOverviewList(@Valid DictQueryParams queryParams) {
        return dictQueries.queryDictOverviewList(queryParams);
    }
}
