package xyz.zhouxy.plusone.system.util;

import javax.annotation.Nonnull;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.crypto.digest.DigestUtil;
import xyz.zhouxy.plusone.exception.PlusoneException;

/**
 * 密码工具类
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
public final class PasswordUtil {
    private static final String SALT_BASE_STRING = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ~`!@#$%^&*()_-+={}[]|\\:;\"',.<>?/";

    /**
     * 将密码和随机盐混合，并进行哈希加密。
     *
     * @param password 密文密码
     * @param salt     随机盐
     * @return 哈希加密的结果
     */
    @Nonnull
    public static String hashPassword(@Nonnull String password, @Nonnull String salt) {
        int length = salt.length();
        int i = length > 0 ? length / 2 : 0;
        var passwordWithSalt = salt.substring(0, i)
                + password
                + salt.substring(1);
        String sha512Hex = DigestUtil.sha512Hex(passwordWithSalt);
        if (sha512Hex == null) {
            throw new PlusoneException(9999999, "未知错误：哈希加密失败！");
        }
        return sha512Hex;
    }

    /**
     * 生成 24 位的字符串
     *
     * @return 生成的随机盐
     */
    public static String generateRandomSalt() {
        return RandomUtil.randomString(SALT_BASE_STRING, 24);
    }

    private PasswordUtil() {
        // 不允许实例化
        throw new IllegalStateException("Utility class");
    }
}
