package xyz.zhouxy.plusone.system.domain.model.account;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Objects;

import lombok.Getter;
import lombok.ToString;
import xyz.zhouxy.plusone.domain.IValueObject;

/**
 * 账号详细信息
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Getter
@ToString
public class AccountInfo implements IValueObject {

    private final Nickname nickname;
    private final URL avatar;
    private final Sex sex;

    private AccountInfo(Nickname nickname, URL avatar, Sex sex) {
        this.nickname = nickname;
        this.avatar = avatar;
        this.sex = Objects.nonNull(sex) ? sex : Sex.UNSET;
    }

    public static AccountInfo of(Nickname nickname, URL avatar, Sex sex) {
        return new AccountInfo(nickname, avatar, sex);
    }

    public static AccountInfo of(String nickname, String avatar, Sex sex) {
        URL avatarURL;
        try {
            avatarURL = Objects.nonNull(avatar) ? new URL(avatar) : null;
        } catch (MalformedURLException e) {
            throw new IllegalArgumentException(e);
        }
        return new AccountInfo(Nickname.ofNullable(nickname), avatarURL, sex);
    }
}
