package xyz.zhouxy.plusone.system.domain.model.account;

import java.util.Objects;

import cn.hutool.core.util.DesensitizedUtil;
import xyz.zhouxy.plusone.constant.RegexConsts;

/**
 * 值对象：电子邮箱地址
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
public class Email extends Principal {

    public static final String REGEX = RegexConsts.EMAIL;

    private Email(String email) {
        super(REGEX);
        if (email == null) {
            throw new IllegalArgumentException("邮箱地址不能为空");
        }
        this.value = email;
        if (!isValid()) {
            throw new IllegalArgumentException("邮箱地址格式错误");
        }
    }

    public static Email of(String email) {
        return new Email(email);
    }

    public static Email ofNullable(String email) {
        return Objects.nonNull(email) ? new Email(email) : null;
    }

    /**
     * 脱敏后的数据
     */
    public String safeValue() {
        return DesensitizedUtil.email(this.value);
    }

    @Override
    public String toString() {
        return this.safeValue();
    }
}
