package xyz.zhouxy.plusone.system.domain.model.menu;

import java.util.Optional;

import com.fasterxml.jackson.annotation.JsonProperty;

import lombok.Getter;
import xyz.zhouxy.plusone.domain.Entity;
import xyz.zhouxy.plusone.domain.IWithLabel;

/**
 * 行为。
 * <p>
 * 一个 Menu 代表对应一个资源，Action 表示对该资源的行为，每个行为具有对应权限。
 * </p>
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 *
 * @see Menu
 */
public class Action extends Entity<Long> implements IWithLabel {

    Long id;
    String resource;
    @Getter
    String identifier;
    @Getter
    String label;

    private Action(Long id, String resource, String identifier, String label) {
        this.id = id;
        this.resource = resource;
        this.identifier = identifier;
        this.label = label;
    }

    static Action of(Long id, String resource, String identifier, String label) {
        return new Action(id, resource, identifier, label);
    }

    static Action newInstance(String resource, String identifier, String label) {
        return new Action(null, resource, identifier, label);
    }

    @JsonProperty("value")
    public String value() {
        return resource + identifier;
    }

    @Override
    public Optional<Long> getId() {
        return Optional.ofNullable(id);
    }
}
