package xyz.zhouxy.plusone.system.domain.model.menu;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import com.fasterxml.jackson.annotation.JsonValue;

import lombok.Getter;
import lombok.ToString;
import xyz.zhouxy.plusone.constant.EntityStatus;
import xyz.zhouxy.plusone.domain.AggregateRoot;
import xyz.zhouxy.plusone.domain.IWithOrderNumber;
import xyz.zhouxy.plusone.domain.IWithVersion;

/**
 * 聚合根：菜单
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 *
 * @see Action
 * @see MenuConstructor
 */
@Getter
@ToString
public class Menu extends AggregateRoot<Long> implements IWithOrderNumber, IWithVersion {

    MenuType type;

    Long id;
    Long parentId;

    String name;
    // 若 type 为 MENU_ITEM 且 path 以 http:// 或 https:// 开头则被识别为外链
    String path;
    String title;
    String icon;
    boolean hidden;
    int orderNumber;
    EntityStatus status;
    String remarks;

    // MENU_ITEM
    String component;
    Boolean cache;
    String resource;
    private List<Action> actions;

    private @Getter long version;

    public void updateMenuInfo(
            MenuType type,
            long parentId,
            String name,
            String path,
            String title,
            String icon,
            boolean hidden,
            int orderNumber,
            EntityStatus status,
            String remarks,
            String component,
            boolean cache,
            String resource) {
        this.type = type;
        this.parentId = parentId;
        this.path = path;
        this.name = name;
        this.title = title;
        this.icon = icon;
        this.hidden = hidden;
        this.orderNumber = orderNumber;
        this.status = status;
        this.component = component;
        this.resource = resource;
        this.cache = cache;
        this.remarks = remarks;
    }

    public Menu addAction(String action, String label) {
        return addAction(Action.newInstance(this.resource, action, label));
    }

    public void removeAction(Long actionId) {
        this.actions.removeIf(action -> Objects.equals(actionId, action.getId().orElseThrow()));
    }

    public void removeAction(String identifier) {
        this.actions.removeIf(action -> Objects.equals(identifier, action.identifier));
    }

    @Override
    public Optional<Long> getId() {
        return Optional.ofNullable(id);
    }

    @Override
    public int getOrderNumber() {
        return this.orderNumber;
    }

    private Menu addAction(Action action) {
        if (this.actions == null) {
            this.actions = new ArrayList<>(8);
        }
        this.actions.add(action);
        return this;
    }

    Menu(MenuType type, Long id, Long parentId, String name, String path, String title, String icon,
            boolean hidden, int orderNumber, EntityStatus status, String remarks, String component, Boolean cache,
            String resource, List<Action> actions, long version) {
        this.type = type;
        this.id = id;
        this.parentId = parentId;
        this.name = name;
        this.path = path;
        this.title = title;
        this.icon = icon;
        this.hidden = hidden;
        this.orderNumber = orderNumber;
        this.status = status;
        this.remarks = remarks;
        this.component = component;
        this.cache = cache;
        this.resource = resource;
        this.actions = actions;
        this.version = version;
    }

    public enum MenuType {
        MENU_LIST, MENU_ITEM;

        @JsonValue
        public int value() {
            return ordinal();
        }
    }
}
