package xyz.zhouxy.plusone.system.domain.model.role;

import java.util.HashSet;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

import lombok.Getter;
import lombok.ToString;
import xyz.zhouxy.plusone.constant.EntityStatus;
import xyz.zhouxy.plusone.domain.AggregateRoot;
import xyz.zhouxy.plusone.domain.IWithVersion;
import xyz.zhouxy.plusone.system.domain.model.menu.Action;
import xyz.zhouxy.plusone.system.domain.model.menu.Menu;

/**
 * 聚合根：角色
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@ToString
public class Role extends AggregateRoot<Long> implements IWithVersion {

    private Long id;
    private @Getter String name;
    private @Getter String identifier;

    private @Getter EntityStatus status;
    private @Getter String remarks;

    private final Set<MenuRef> menus = new HashSet<>();
    private final Set<ActionRef> permissions = new HashSet<>();

    private @Getter long version;

    public void bindMenus(Set<MenuRef> menus) {
        this.menus.clear();
        this.menus.addAll(menus);
    }

    public void bindPermissions(Set<ActionRef> permissions) {
        this.permissions.clear();
        this.permissions.addAll(permissions);
    }

    public void update(
            String name,
            String identifier,
            EntityStatus status,
            String remarks,
            Set<Menu> menus,
            Set<Action> permissions) {
        this.name = name;
        this.identifier = identifier;
        this.status = status;
        this.remarks = remarks;
        bindMenus(menus.stream().map(menu -> new MenuRef(menu.getId().orElseThrow())).collect(Collectors.toSet()));
        bindPermissions(permissions
                .stream()
                .map(permission -> new ActionRef(permission.getId().orElseThrow()))
                .collect(Collectors.toSet()));
    }

    // getters

    @Override
    public Optional<Long> getId() {
        return Optional.ofNullable(id);
    }

    public Set<MenuRef> getMenus() {
        return Set.copyOf(menus);
    }

    public Set<ActionRef> getPermissions() {
        return Set.copyOf(permissions);
    }

    public static Role newInstance(
            String name,
            String identifier,
            EntityStatus status,
            String remarks,
            Set<MenuRef> menus,
            Set<ActionRef> permissions) {
        return new Role(null, name, identifier, status, remarks, menus, permissions, 0L);
    }

    /**
     * 构造方法
     *
     * @param id          id
     * @param name        角色名
     * @param identifier  标识符
     * @param status      状态
     * @param remarks     备注
     * @param menus       菜单
     * @param permissions 权限
     * @param version     版本号
     */
    Role(Long id, String name, String identifier, EntityStatus status, String remarks,
            Set<MenuRef> menus, Set<ActionRef> permissions, long version) {
        this.id = id;
        this.name = name;
        this.identifier = identifier;
        this.status = status;
        this.remarks = remarks;
        bindMenus(menus);
        bindPermissions(permissions);
        this.version = version;
    }
}
