package xyz.zhouxy.plusone.system.domain.service;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.stereotype.Service;

import xyz.zhouxy.plusone.system.domain.model.menu.Menu;
import xyz.zhouxy.plusone.system.domain.model.menu.MenuRepository;
import xyz.zhouxy.plusone.system.domain.model.role.MenuRef;
import xyz.zhouxy.plusone.system.domain.model.role.Role;
import xyz.zhouxy.plusone.system.domain.model.role.RoleRepository;

/**
 * 领域服务：菜单服务
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@Service
public class MenuService {

    private final RoleRepository roleRepository;
    private final MenuRepository menuRepository;

    MenuService(RoleRepository roleRepository, MenuRepository menuRepository) {
        this.roleRepository = roleRepository;
        this.menuRepository = menuRepository;
    }

    /**
     * 根据账号 id 查询菜单列表（不形成树结构）
     * @param accountId
     * @return
     */
    public Collection<Menu> queryAllMenuListByAccountId(Long accountId) {
        Collection<Role> roles = roleRepository.findByAccountId(accountId);
        Set<MenuRef> menuRefs = new HashSet<>();
        roles.forEach(role -> menuRefs.addAll(role.getMenus()));
        return menuRepository.findByIdIn(menuRefs.stream().map(MenuRef::menuId).collect(Collectors.toSet()));
    }

}
