package xyz.zhouxy.plusone.system.domain.model.account;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;

import xyz.zhouxy.plusone.util.AssertResult;
import xyz.zhouxy.plusone.util.NumberUtil;

class AccountRoleRefDAO {
    private final NamedParameterJdbcTemplate jdbc;

    AccountRoleRefDAO(NamedParameterJdbcTemplate jdbc) {
        this.jdbc = jdbc;
    }

    Set<Long> selectRoleIdsByAccountId(Long accountId) {
        List<Long> roleRefs = this.jdbc.queryForList("""
                SELECT r.id FROM sys_role r RIGHT JOIN sys_account_role ar ON r.id = ar.role_id
                WHERE r.deleted = 0 AND ar.account_id = :accountId;
                """,
                new MapSqlParameterSource("accountId", accountId),
                Long.TYPE);
        return new HashSet<>(roleRefs);
    }

    void clearAccountRoleRefs(Account entity) {
        var param = new MapSqlParameterSource("accountId", entity.getId().orElseThrow());
        this.jdbc.update("DELETE FROM sys_account_role WHERE account_id = :accountId", param);
    }

    void insertAccountRoleRefs(Long accountId, Set<Long> roleRefs) {
        String sql = "INSERT INTO sys_account_role (account_id, role_id) VALUES (:accountId, :roleId)";
        MapSqlParameterSource[] batchArgs = roleRefs
                .stream()
                .map((Long roleId) -> new MapSqlParameterSource()
                        .addValue("accountId", accountId)
                        .addValue("roleId", roleId))
                .toArray(MapSqlParameterSource[]::new);
        int[] i = this.jdbc.batchUpdate(sql, batchArgs);
        AssertResult.update(roleRefs.size(), NumberUtil.sum(i));
    }

    void saveAccountRoleRefs(Account entity) {
        Long accountId = entity.getId().orElseThrow();
        Set<Long> roleRefs = entity.getRoleIds();
        clearAccountRoleRefs(entity);
        insertAccountRoleRefs(accountId, roleRefs);
    }
}
