package xyz.zhouxy.plusone.system.domain.model.dict;

import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.util.CollectionUtils;

import xyz.zhouxy.plusone.util.AssertResult;
import xyz.zhouxy.plusone.util.NumberUtil;

class DictValueDAO {
    private final NamedParameterJdbcTemplate jdbc;

    DictValueDAO(NamedParameterJdbcTemplate jdbc) {
        this.jdbc = jdbc;
    }

    void updateDictValues(Dict entity) {
        MapSqlParameterSource deleteParam = new MapSqlParameterSource("dictType", entity.getId().orElseThrow());
        this.jdbc.update("DELETE FROM sys_dict_value WHERE dict_type = :dictType", deleteParam);
        int i = insertDictValues(entity.getId().orElseThrow(), entity);
        AssertResult.update(i, entity.count());
    }

    int insertDictValues(Long dictId, Dict entity) {
        if (Objects.isNull(dictId) || Objects.isNull(entity) || CollectionUtils.isEmpty(entity.getValues())) {
            return 0;
        }
        int[] i = this.jdbc.batchUpdate(
                "INSERT INTO sys_dict_value (dict_type, dict_key, label) VALUES (:dictType, :dictKey, :label)",
                entity.getValues().stream()
                        .map(dictValue -> new MapSqlParameterSource()
                                .addValue("dictType", dictId)
                                .addValue("dictKey", dictValue.getKey())
                                .addValue("label", dictValue.getLabel()))
                        .toArray(SqlParameterSource[]::new));
        return NumberUtil.sum(i);
    }

    Set<DictValue> selectDictValuesByDictId(long id) {
        return this.jdbc.queryForStream("""
                SELECT dict_key, label FROM sys_dict_value WHERE dict_type = :dictType
                """, new MapSqlParameterSource("dictType", id),
                (rs, rowNum) -> DictValue.of(rs.getInt("dict_key"), rs.getString("label")))
                .collect(Collectors.toSet());
    }
}
