package xyz.zhouxy.plusone.system.domain.model.role;

import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;

import xyz.zhouxy.plusone.util.AssertResult;
import xyz.zhouxy.plusone.util.NumberUtil;

class RoleMenuRefDAO {

    private final NamedParameterJdbcTemplate jdbc;

    public RoleMenuRefDAO(NamedParameterJdbcTemplate jdbc) {
        this.jdbc = jdbc;
    }

    Set<MenuRef> selectMenuRefsByRoleId(long roleId) {
        return this.jdbc.queryForList("SELECT menu_id FROM sys_role_menu WHERE role_id = :roleId",
                new MapSqlParameterSource("roleId", roleId),
                Long.TYPE)
                .stream()
                .map(MenuRef::of)
                .collect(Collectors.toSet());
    }

    void clearRoleMenuRefs(Role entity) {
        MapSqlParameterSource param = new MapSqlParameterSource("roleId", entity.getId().orElseThrow());
        this.jdbc.update("DELETE FROM sys_role_menu WHERE role_id = :roleId", param);
    }

    void saveRoleMenuRefs(Long roleId, Role entity) {
        String sql = "INSERT INTO sys_role_menu(role_id, menu_id) VALUES (:roleId, :menuId)";
        MapSqlParameterSource[] batchArgs = entity.getMenus()
                .stream()
                .map(menuRef -> new MapSqlParameterSource()
                        .addValue("roleId", roleId)
                        .addValue("menuId", menuRef.menuId()))
                .toArray(MapSqlParameterSource[]::new);
        int[] i = this.jdbc.batchUpdate(sql, batchArgs);
        AssertResult.update(entity.getMenus().size(), NumberUtil.sum(i));
    }
}
