package xyz.zhouxy.plusone.system.domain.model.role;

import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;

import xyz.zhouxy.plusone.util.AssertResult;
import xyz.zhouxy.plusone.util.NumberUtil;

class RolePermissionRefDAO {
    private final NamedParameterJdbcTemplate jdbc;

    public RolePermissionRefDAO(NamedParameterJdbcTemplate jdbc) {
        this.jdbc = jdbc;
    }

    Set<ActionRef> selectPermissionRefsByRoleId(long roleId) {
        return this.jdbc.queryForList("SELECT permission_id FROM sys_role_permission WHERE role_id = :roleId",
                new MapSqlParameterSource("roleId", roleId),
                Long.TYPE)
                .stream()
                .map(ActionRef::of)
                .collect(Collectors.toSet());
    }

    void clearRolePermissionRefs(Role entity) {
        MapSqlParameterSource param = new MapSqlParameterSource("roleId", entity.getId().orElseThrow());
        this.jdbc.update("DELETE FROM sys_role_permission WHERE role_id = :roleId", param);
    }

    void saveRolePermissionRefs(Long roleId, Role entity) {
        String sql = "INSERT INTO sys_role_permission(role_id, permission_id) VALUES (:roleId, :permissionId)";
        SqlParameterSource[] batchArgs = entity.getPermissions()
                .stream()
                .map(menuRef -> new MapSqlParameterSource()
                        .addValue("roleId", roleId)
                        .addValue("permissionId", menuRef.actionId()))
                .toArray(MapSqlParameterSource[]::new);
        int[] i = this.jdbc.batchUpdate(sql, batchArgs);
        AssertResult.update(entity.getMenus().size(), NumberUtil.sum(i));
    }
}
